@@ -46,6 +46,9 @@
 /* */
 #undef HAVE_STRLEN
 
+/* */
+#undef HAVE_STRNLEN
+
 /* */
 #undef HAVE_STRCPY
 
@@ -79,6 +82,9 @@
 /* */
 #undef HAVE_MEMCHR
 
+/* */
+#undef HAVE_MEMRCHR
+
 /* */
 #undef HAVE_MEMCPY
 
@@ -100,6 +106,9 @@
 /* */
 #undef HAVE_SIGSETJMP
 
+/* */
+#undef HAVE_GCC_ASSEMBLER
+
 /* */
 #undef HAVE___BUILTIN_ADD_OVERFLOW
 
@@ -214,6 +223,9 @@
 /* */
 #undef GRDRV_GRX
 
+/* Have freetype */
+#undef HAVE_FREETYPE
+
 /* Jpeg by Clock */
 #undef HAVE_JPEG
 
