@@ -20,11 +20,11 @@
 
 #ifdef GRDRV_DIRECTFB
 
-#include <netinet/in.h>  /* for htons */
+#include "links.h"
 
+#undef debug
 #include <directfb.h>
 
-#include "links.h"
 #include "dfb_cur.h"
 
 
@@ -61,8 +61,8 @@ static void directfb_flip_surface      (void *pointer);
 static void directfb_check_events      (void *pointer);
 static void directfb_translate_key     (DFBWindowEvent *event,
                                         int *key, int *flag);
-static void directfb_add_to_table      (struct graphics_device *gd);
-static void directfb_remove_from_table (struct graphics_device *gd);
+static void directfb_add_to_table      (struct graphics_device *dev);
+static void directfb_remove_from_table (struct graphics_device *dev);
 static struct graphics_device * directfb_lookup_in_table (DFBWindowID  id);
 
 
@@ -99,7 +99,7 @@ directfb_init_driver (unsigned char *param, unsigned char *display)
 	directfb_driver.depth = 196;
 
   /* endian test */
-  if (htons (0x1234) == 0x1234) {
+  if (big_endian) {
      if ((directfb_driver.depth & 0x7) == 2)
 	directfb_driver.depth |= 0x100;
      if ((directfb_driver.depth & 0x7) == 4)
@@ -142,7 +142,7 @@ ret:
 static struct graphics_device *
 directfb_init_device (void)
 {
-  struct graphics_device *gd;
+  struct graphics_device *dev;
   DFBDeviceData          *data;
   IDirectFBWindow        *window;
   DFBWindowDescription    desc;
@@ -160,13 +160,13 @@ directfb_init_device (void)
     return NULL;
   }
 
-  gd = mem_alloc (sizeof (struct graphics_device));
+  dev = mem_alloc (sizeof (struct graphics_device));
 
-  gd->size.x1 = 0;
-  gd->size.y1 = 0;
-  window->GetSize (window, &gd->size.x2, &gd->size.y2);
+  dev->size.x1 = 0;
+  dev->size.y1 = 0;
+  window->GetSize (window, &dev->size.x2, &dev->size.y2);
 
-  gd->clip = gd->size;
+  dev->clip = dev->size;
 
   data = mem_alloc (sizeof (DFBDeviceData));
 
@@ -179,37 +179,37 @@ directfb_init_device (void)
   window->GetSurface (window, &data->surface);
   window->GetID (window, &data->id);
 
-  gd->driver_data = data;
-  gd->user_data   = NULL;
+  dev->driver_data = data;
+  dev->user_data   = NULL;
 
-  directfb_add_to_table (gd);
+  directfb_add_to_table (dev);
 
   window->AttachEventBuffer (window, events);
 
   window->SetOpacity (window, FOCUSED_OPACITY);
 
-  return gd;
+  return dev;
 }
 
 static void
-directfb_shutdown_device (struct graphics_device *gd)
+directfb_shutdown_device (struct graphics_device *dev)
 {
   DFBDeviceData *data;
 
-  if (!gd)
+  if (!dev)
     return;
 
-  data = gd->driver_data;
+  data = dev->driver_data;
 
   unregister_bottom_half (directfb_flip_surface, data);
-  directfb_remove_from_table (gd);
+  directfb_remove_from_table (dev);
 
   data->surface->Release (data->surface);
   data->window->Destroy (data->window);
   data->window->Release (data->window);
 
   mem_free (data);
-  mem_free (gd);
+  mem_free (dev);
 }
 
 static void
@@ -234,12 +234,6 @@ directfb_shutdown_driver (void)
   dfb = NULL;
 }
 
-static unsigned char *
-directfb_get_driver_param (void)
-{
-  return NULL;
-}
-
 static int
 directfb_get_empty_bitmap (struct bitmap *bmp)
 {
@@ -307,15 +301,14 @@ directfb_unregister_bitmap (struct bitmap *bmp)
 }
 
 static void
-directfb_draw_bitmap (struct graphics_device *gd, struct bitmap *bmp,
+directfb_draw_bitmap (struct graphics_device *dev, struct bitmap *bmp,
                       int x, int y)
 {
-  DFBDeviceData    *data = gd->driver_data;
+  DFBDeviceData    *data = dev->driver_data;
   IDirectFBSurface *src  = bmp->flags;
   if (!src) return;
 
-  if (gd->clip.x1 >= gd->clip.x2 ||
-      gd->clip.y1 >= gd->clip.y2) return;
+  CLIP_DRAW_BITMAP
 
   data->surface->Blit (data->surface, src, NULL, x, y);
 
@@ -339,12 +332,16 @@ static inline void directfb_set_color (IDirectFBSurface *surface, long color)
 }
 
 static void
-directfb_fill_area (struct graphics_device *gd,
+directfb_fill_area (struct graphics_device *dev,
                     int x1, int y1, int x2, int y2, long color)
 {
-  DFBDeviceData *data = gd->driver_data;
-  int w = x2 - x1;
-  int h = y2 - y1;
+  DFBDeviceData *data = dev->driver_data;
+  int w, h;
+
+  CLIP_FILL_AREA
+
+  w = x2 - x1;
+  h = y2 - y1;
 
   directfb_set_color (data->surface, color);
   data->surface->FillRectangle (data->surface, x1, y1, w, h);
@@ -353,91 +350,64 @@ directfb_fill_area (struct graphics_device *gd,
 }
 
 static void
-directfb_draw_hline (struct graphics_device *gd,
-                     int left, int y, int right, long color)
+directfb_draw_hline (struct graphics_device *dev,
+                     int x1, int y, int x2, long color)
 {
-  DFBDeviceData *data = gd->driver_data;
+  DFBDeviceData *data = dev->driver_data;
 
-  if (right <= left) return;
+  CLIP_DRAW_HLINE
 
-  right--;
+  x2--;
 
   directfb_set_color (data->surface, color);
-  data->surface->DrawLine (data->surface, left, y, right, y);
+  data->surface->DrawLine (data->surface, x1, y, x2, y);
 
-  directfb_register_flip (data, left, y, right - left, 1);
+  directfb_register_flip (data, x1, y, x2 - x1, 1);
 }
 
 static void
-directfb_draw_vline (struct graphics_device *gd,
-                     int x, int top, int bottom, long color)
+directfb_draw_vline (struct graphics_device *dev,
+                     int x, int y1, int y2, long color)
 {
-  DFBDeviceData *data = gd->driver_data;
+  DFBDeviceData *data = dev->driver_data;
 
-  if (bottom <= top) return;
+  CLIP_DRAW_VLINE
 
-  bottom--;
+  y2--;
 
   directfb_set_color (data->surface, color);
-  data->surface->DrawLine (data->surface, x, top, x, bottom);
+  data->surface->DrawLine (data->surface, x, y1, x, y2);
 
-  directfb_register_flip (data, x, top, 1, bottom - top);
+  directfb_register_flip (data, x, y1, 1, y2 - y1);
 }
 
 static void
-directfb_set_clip_area (struct graphics_device *gd, struct rect *r)
+directfb_set_clip_area (struct graphics_device *dev)
 {
-  DFBDeviceData *data   = gd->driver_data;
+  DFBDeviceData *data   = dev->driver_data;
   DFBRegion      region;
-  generic_set_clip_area(gd, r);
-  region.x1 = gd->clip.x1;
-  region.y1 = gd->clip.y1;
-  region.x2 = gd->clip.x2 - 1;
-  region.y2 = gd->clip.y2 - 1;
+
+  region.x1 = dev->clip.x1;
+  region.y1 = dev->clip.y1;
+  region.x2 = dev->clip.x2 - 1;
+  region.y2 = dev->clip.y2 - 1;
 
   data->surface->SetClip (data->surface, &region);
 }
 
 static int
-directfb_hscroll (struct graphics_device *gd, struct rect_set **set, int sc)
+directfb_scroll (struct graphics_device *dev, struct rect_set **set, int scx, int scy)
 {
-  DFBDeviceData *data = gd->driver_data;
+  DFBDeviceData *data = dev->driver_data;
   DFBRectangle   rect;
 
-  *set = NULL;
-  if (!sc)
-    return 0;
-
-  rect.x = gd->clip.x1;
-  rect.y = gd->clip.y1;
-  rect.w = gd->clip.x2 - rect.x;
-  rect.h = gd->clip.y2 - rect.y;
+  rect.x = dev->clip.x1;
+  rect.y = dev->clip.y1;
+  rect.w = dev->clip.x2 - rect.x;
+  rect.h = dev->clip.y2 - rect.y;
 
   data->surface->Blit (data->surface,
-                       data->surface, &rect, rect.x + sc, rect.y);
-
-  directfb_register_flip (data, rect.x, rect.y, rect.w, rect.h);
-
-  return 1;
-}
-
-static int
-directfb_vscroll (struct graphics_device *gd, struct rect_set **set, int sc)
-{
-  DFBDeviceData *data = gd->driver_data;
-  DFBRectangle   rect;
-
-  *set = NULL;
-  if (!sc)
-    return 0;
-
-  rect.x = gd->clip.x1;
-  rect.y = gd->clip.y1;
-  rect.w = gd->clip.x2 - rect.x;
-  rect.h = gd->clip.y2 - rect.y;
-
-  data->surface->Blit (data->surface,
-                       data->surface, &rect, rect.x, rect.y + sc);
+                       data->surface, &rect, rect.x + scx, rect.y + scy);
 
   directfb_register_flip (data, rect.x, rect.y, rect.w, rect.h);
 
@@ -487,9 +457,9 @@ directfb_flip_surface (void *pointer)
 }
 
 static void
-directfb_flush(struct graphics_device *gd)
+directfb_flush(struct graphics_device *dev)
 {
-  DFBDeviceData *data = gd->driver_data;
+  DFBDeviceData *data = dev->driver_data;
   unregister_bottom_half (directfb_flip_surface, data);
   directfb_flip_surface(data);
 }
@@ -497,7 +467,7 @@ directfb_flush(struct graphics_device *gd)
 static void
 directfb_check_events (void *pointer)
 {
-  struct graphics_device *gd   = NULL;
+  struct graphics_device *dev   = NULL;
   DFBDeviceData          *data = NULL;
   DFBWindowEvent          event;
   DFBWindowEvent          next;
@@ -520,14 +490,14 @@ directfb_check_events (void *pointer)
           continue;
         }
 
-      if (!data || data->id != event.window_id)
+      if (!dev || data->id != event.window_id)
         {
-          gd = directfb_lookup_in_table (event.window_id);
-          if (!gd)
+          dev = directfb_lookup_in_table (event.window_id);
+          if (!dev)
             continue;
         }
 
-      data = gd->driver_data;
+      data = dev->driver_data;
 
       switch (event.type)
         {
@@ -548,9 +518,9 @@ directfb_check_events (void *pointer)
                  (next.window_id == data->id))
             events->GetEvent (events, DFB_EVENT (&event));
 
-          gd->size.x2 = event.w;
-          gd->size.y2 = event.h;
-          gd->resize_handler (gd);
+          dev->size.x2 = event.w;
+          dev->size.y2 = event.h;
+          dev->resize_handler (dev);
           break;
 
         case DWET_KEYDOWN:
@@ -559,7 +529,7 @@ directfb_check_events (void *pointer)
 
             directfb_translate_key (&event, &key, &flag);
             if (key)
-              gd->keyboard_handler (gd, key, flag);
+              dev->keyboard_handler (dev, key, flag);
           }
           break;
 
@@ -602,12 +572,12 @@ directfb_check_events (void *pointer)
                 continue;
               }
 
-            gd->mouse_handler (gd, event.x, event.y, flags);
+            dev->mouse_handler (dev, event.x, event.y, flags);
           }
           break;
 
         case DWET_WHEEL:
-          gd->mouse_handler (gd, event.x, event.y,
+          dev->mouse_handler (dev, event.x, event.y,
                              B_MOVE |
                              (event.step > 0 ? B_WHEELUP : B_WHEELDOWN));
 	  break;
@@ -637,12 +607,12 @@ directfb_check_events (void *pointer)
                 break;
               }
 
-            gd->mouse_handler (gd, event.x, event.y, flags);
+            dev->mouse_handler (dev, event.x, event.y, flags);
           }
           break;
 
         case DWET_CLOSE:
-          gd->keyboard_handler (gd, KBD_CLOSE, 0);
+          dev->keyboard_handler (dev, KBD_CLOSE, 0);
           break;
 
         default:
@@ -712,9 +682,9 @@ directfb_translate_key (DFBWindowEvent *event, int *key, int *flag)
 }
 
 static void
-directfb_add_to_table (struct graphics_device *gd)
+directfb_add_to_table (struct graphics_device *dev)
 {
-  DFBDeviceData           *data = gd->driver_data;
+  DFBDeviceData           *data = dev->driver_data;
   struct graphics_device **devices;
   int i;
 
@@ -731,13 +701,13 @@ directfb_add_to_table (struct graphics_device *gd)
 
       if ((unsigned)c > MAXINT / sizeof(void *) - 1) overalloc();
       devices = mem_realloc (devices, (c + 1) * sizeof (void *));
-      devices[c-1] = gd;
+      devices[c-1] = dev;
       devices[c]   = NULL;
     }
   else
     {
       devices = mem_alloc (2 * sizeof (void *));
-      devices[0] = gd;
+      devices[0] = dev;
       devices[1] = NULL;
     }
 
@@ -745,9 +715,9 @@ directfb_add_to_table (struct graphics_device *gd)
 }
 
 static void
-directfb_remove_from_table (struct graphics_device *gd)
+directfb_remove_from_table (struct graphics_device *dev)
 {
-  DFBDeviceData           *data = gd->driver_data;
+  DFBDeviceData           *data = dev->driver_data;
   struct graphics_device **devices;
   int i, j, c;
 
@@ -758,7 +728,7 @@ directfb_remove_from_table (struct graphics_device *gd)
     return;
 
   for (j = 0, c = -1; devices[j]; j++)
-    if (devices[j] == gd)
+    if (devices[j] == dev)
       c = j;
 
   if (c < 0)
@@ -804,7 +774,7 @@ struct graphics_driver directfb_driver =
   directfb_shutdown_driver,
   NULL,
   NULL,
-  directfb_get_driver_param,
+  NULL,
   NULL,
   NULL,
   NULL,
@@ -818,12 +788,13 @@ struct graphics_driver directfb_driver =
   directfb_fill_area,
   directfb_draw_hline,
   directfb_draw_vline,
-  directfb_hscroll,
-  directfb_vscroll,
+  directfb_scroll,
   directfb_set_clip_area,
   directfb_flush,
-  dummy_block,
-  dummy_unblock,
+  NULL,	 /*  block */
+  NULL,	 /*  unblock */
+  NULL,	 /*  set_palette */
+  NULL,	 /*  get_real_colors */
   NULL,	 /*  set_title  */
   NULL,	 /*  exec	*/
   NULL,	 /*  set_clipboard_text */
@@ -831,8 +802,7 @@ struct graphics_driver directfb_driver =
   0,	 /*  depth      */
   0, 0,	 /*  size       */
   GD_UNICODE_KEYS | GD_NO_OS_SHELL | GD_NOAUTO, /*  flags      */
-  0,     /*  codepage   */
-  NULL,	 /*  shell	*/
+  NULL,	 /*  param	*/
 };
 
 #endif /* GRDRV_DIRECTFB */
