@@ -8,8 +8,6 @@
        protoze pozadi obrazku musi byt stejne jako pozadi stranky, a to se nikdy
        neditheruje, protoze je to vzdy jednolita barva. Kdyz uz to nepujde
        odstranit tak tam aspon dat fixne zaokrouhlovani.
- TODO: pouzit get_filled_bitmap az bude napsany k optimalizaci zadavani
-       grafickych dat do X serveru z hlediska zaalokovane pameti.
  TODO: dodelat stripy do jpegu a png a tiff.
  */
 
@@ -68,7 +66,6 @@ static void destroy_decoder (struct cached_image *cimg)
 static void mem_free_buffer(struct cached_image *cimg)
 {
 	mem_free(cimg->buffer);
-	mem_freed_large(cimg->width * cimg->height * cimg->buffer_bytes_per_pixel);
 }
 
 static void img_destruct_image(struct g_object *object)
@@ -124,8 +121,8 @@ void img_destruct_cached_image(struct cached_image *cimg)
 
 #ifdef DEBUG
 		default:
-		fprintf(stderr,"img_destruct_cached_image: state=%d\n",cimg->state);
-		internal("Invalid state in struct cached_image");
+		fprintf(stderr, "img_destruct_cached_image: state=%d\n", cimg->state);
+		internal_error("Invalid state in struct cached_image");
 #endif /* #ifdef DEBUG */
 	}
 	mem_free(cimg->url);
@@ -191,18 +188,14 @@ static int height2width(double height_px, double width_mm, double height_mm)
 
 /* Compute 8-bit background for filling buffer with cimg->*_gamma
  * (performs rounding) */
-void compute_background_8(unsigned char *rgb, struct cached_image *cimg)
+void compute_background_8(struct cached_image *cimg, unsigned char rgb[3])
 {
 	unsigned short red, green, blue;
 
-	round_color_sRGB_to_48(&red, &green, &blue
-		, cimg->background_color);
-	rgb[0]=ags_16_to_8(red
-		,(float)(cimg->red_gamma/(float)user_gamma));
-	rgb[1]=ags_16_to_8(green
-		,(float)(cimg->green_gamma/(float)user_gamma));
-	rgb[2]=ags_16_to_8(blue
-		,(float)(cimg->blue_gamma/(float)user_gamma));
+	round_color_sRGB_to_48(&red, &green, &blue, cimg->background_color);
+	rgb[0] = ags_16_to_8(red, (float)(cimg->red_gamma/(float)(user_gamma tcc_hack)));
+	rgb[1] = ags_16_to_8(green, (float)(cimg->green_gamma/(float)(user_gamma tcc_hack)));
+	rgb[2] = ags_16_to_8(blue, (float)(cimg->blue_gamma/(float)(user_gamma tcc_hack)));
 }
 
 /* updates cimg state when header dimensions are know. Only allowed to be called
@@ -235,7 +228,7 @@ int header_dimensions_known(struct cached_image *cimg)
 #ifdef DEBUG
 	if ((cimg->state^8)&13){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in header_dimensions_known");
+		internal_error("Invalid state in header_dimensions_known");
 	}
 #endif /* #ifdef DEBUG */
 	if (cimg->width<1||cimg->height<1){
@@ -321,7 +314,7 @@ int header_dimensions_known(struct cached_image *cimg)
 #ifdef DEBUG
 		if (cimg->height<=0){
 			fprintf(stderr,"cimg->height=%d\n",cimg->height);
-			internal("Invalid cimg->height in strip_optimized section of\
+			internal_error("Invalid cimg->height in strip_optimized section of\
  header_dimensions_known");
 		}
 #endif /* #ifdef DEBUG */
@@ -356,8 +349,7 @@ int header_dimensions_known(struct cached_image *cimg)
 		cimg->bmp_used=0;
 		if (cimg->width && (unsigned)cimg->width * (unsigned)cimg->height / (unsigned)cimg->width != (unsigned)cimg->height) overalloc();
 		if ((unsigned)cimg->width * (unsigned)cimg->height > (unsigned)MAXINT / cimg->buffer_bytes_per_pixel) overalloc();
-		cimg->buffer=mem_alloc_mayfail(cimg->width*cimg->height
-			*cimg->buffer_bytes_per_pixel);
+		cimg->buffer=mem_alloc_mayfail((size_t)cimg->width * (size_t)cimg->height * (size_t)cimg->buffer_bytes_per_pixel);
 		if (!cimg->buffer)
 			return 1;
 		if (cimg->buffer_bytes_per_pixel==4
@@ -365,34 +357,25 @@ int header_dimensions_known(struct cached_image *cimg)
 				*sizeof(unsigned short))
 			{
 			/* Make the buffer contain full transparency */
-			memset(cimg->buffer,0,cimg->width*cimg->height
-				*cimg->buffer_bytes_per_pixel);
+			memset(cimg->buffer, 0, (size_t)cimg->width * (size_t)cimg->height * (size_t)cimg->buffer_bytes_per_pixel);
 		}else{
 			/* Fill the buffer with background color */
-			if (cimg->buffer_bytes_per_pixel>4){
+			if (cimg->buffer_bytes_per_pixel > 4){
 				/* 16-bit */
 				unsigned short red, green, blue;
 
-				round_color_sRGB_to_48(&red, &green, &blue
-					, cimg->background_color);
+				round_color_sRGB_to_48(&red, &green, &blue, cimg->background_color);
 
-				red=ags_16_to_16(red
-					,(float)(cimg->red_gamma/(float)user_gamma));
-				green=ags_16_to_16(green
-					,(float)(cimg->green_gamma/(float)user_gamma));
-				blue=ags_16_to_16(blue
-					,(float)(cimg->blue_gamma/(float)user_gamma));
-				mix_one_color_48((unsigned short *)cimg->buffer
-					,cimg->width*cimg->height,red
-					,green, blue);
-			}else{
+				red=ags_16_to_16(red, (float)(cimg->red_gamma / (float)(user_gamma tcc_hack)));
+				green=ags_16_to_16(green, (float)(cimg->green_gamma / (float)(user_gamma tcc_hack)));
+				blue=ags_16_to_16(blue, (float)(cimg->blue_gamma / (float)(user_gamma tcc_hack)));
+				mix_one_color_48((unsigned short *)cimg->buffer, cimg->width*cimg->height, red, green, blue);
+			} else {
 				unsigned char rgb[3];
 
 				/* 8-bit */
-				compute_background_8(rgb,cimg);
-				mix_one_color_24(cimg->buffer
-					,cimg->width*cimg->height
-					,rgb[0],rgb[1],rgb[2]);
+				compute_background_8(cimg, rgb);
+				mix_one_color_24(cimg->buffer, cimg->width * cimg->height, rgb[0], rgb[1], rgb[2]);
 			}
 		}
 	}
@@ -408,89 +391,69 @@ int header_dimensions_known(struct cached_image *cimg)
 /* Fills "tmp" buffer with the resulting data and does not free the input
  * buffer. May be called only in states 12 and 14 of cimg
  */
-static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *cimg
-	,unsigned char *buffer, int height)
+static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *cimg, unsigned char *buffer, int height)
 {
-	unsigned short red, green,blue;
+	unsigned short red, green, blue;
 
 #ifdef DEBUG
-	if (cimg->state!=12&&cimg->state!=14){
-		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("invalid state in buffer_to_16");
+	if (cimg->state != 12 && cimg->state != 14){
+		fprintf(stderr, "cimg->state=%d\n", cimg->state);
+		internal_error("invalid state in buffer_to_16");
 	}
 #endif /* #ifdef DEBUG */
-	switch (cimg->buffer_bytes_per_pixel){
+	switch (cimg->buffer_bytes_per_pixel) {
 		case 3:
-			if (cimg->gamma_table){
-				agx_24_to_48_table(tmp, buffer,
-					cimg->width*height
-					,cimg->gamma_table);
-			}
-			else{
-				agx_24_to_48(tmp,buffer,cimg->width
-					*height
-					,(float)((float)user_gamma/cimg->red_gamma)
-					,(float)((float)user_gamma/cimg->green_gamma)
-					,(float)((float)user_gamma/cimg->blue_gamma));
+			if (cimg->gamma_table) {
+				agx_24_to_48_table(tmp, buffer, cimg->width*height, cimg->gamma_table);
+			} else {
+				agx_24_to_48(tmp, buffer, cimg->width * height,
+					(float)((float)(user_gamma tcc_hack) / cimg->red_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->green_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->blue_gamma));
 			}
 		break;
 
-		case 3*sizeof(unsigned short):
-			if (cimg->gamma_table){
-				agx_48_to_48_table(tmp
-					,(unsigned short *)buffer
-					,cimg->width*height, cimg->gamma_table);
-			}else{
-				agx_48_to_48(tmp,(unsigned short *)buffer
-					,cimg->width*height
-					,(float)((float)user_gamma/cimg->red_gamma)
-					,(float)((float)user_gamma/cimg->green_gamma)
-					,(float)((float)user_gamma/cimg->blue_gamma));
+		case 3 * sizeof(unsigned short):
+			if (cimg->gamma_table) {
+				agx_48_to_48_table(tmp, (unsigned short *)buffer, cimg->width * height, cimg->gamma_table);
+			} else {
+				agx_48_to_48(tmp, (unsigned short *)buffer, cimg->width * height,
+					(float)((float)(user_gamma tcc_hack) / cimg->red_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->green_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->blue_gamma));
 			}
 		break;
 
 		/* Alpha's: */
 		case 4:
-		{
-
 			round_color_sRGB_to_48(&red,&green,&blue,cimg->background_color);
-			if (cimg->gamma_table){
-				agx_and_uc_32_to_48_table(
-						tmp, buffer, cimg->width *height,
-						cimg->gamma_table, red, green, blue);
-			}else{
-				agx_and_uc_32_to_48(tmp,buffer
-					,cimg->width*height
-					,(float)((float)user_gamma/cimg->red_gamma)
-					,(float)((float)user_gamma/cimg->green_gamma)
-					,(float)((float)user_gamma/cimg->blue_gamma)
-					,red, green, blue);
+			if (cimg->gamma_table) {
+				agx_and_uc_32_to_48_table(tmp, buffer, cimg->width * height, cimg->gamma_table, red, green, blue);
+			} else {
+				agx_and_uc_32_to_48(tmp,buffer, cimg->width*height,
+					(float)((float)(user_gamma tcc_hack) / cimg->red_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->green_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->blue_gamma),
+					red, green, blue);
 			}
-		}
 		break;
 
-		case 4*sizeof(unsigned short):
-		{
-			round_color_sRGB_to_48(&red, &green, &blue,
-				cimg->background_color);
-			if (cimg->gamma_table){
-				agx_and_uc_64_to_48_table
-					(tmp, (unsigned short *)buffer, cimg->width*height
-					,cimg->gamma_table, red, green, blue);
-			}else{
-				agx_and_uc_64_to_48(tmp
-					,(unsigned short*)buffer,cimg->width*height
-					,(float)((float)user_gamma/cimg->red_gamma)
-					,(float)((float)user_gamma/cimg->green_gamma)
-					,(float)((float)user_gamma/cimg->blue_gamma)
-					,red,green,blue);
+		case 4 * sizeof(unsigned short):
+			round_color_sRGB_to_48(&red, &green, &blue, cimg->background_color);
+			if (cimg->gamma_table) {
+				agx_and_uc_64_to_48_table(tmp, (unsigned short *)buffer, cimg->width * height, cimg->gamma_table, red, green, blue);
+			} else {
+				agx_and_uc_64_to_48(tmp, (unsigned short *)buffer, cimg->width * height,
+					(float)((float)(user_gamma tcc_hack) / cimg->red_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->green_gamma),
+					(float)((float)(user_gamma tcc_hack) / cimg->blue_gamma),
+					red,green,blue);
 			}
-		}
 		break;
 
 #ifdef DEBUG
 		default:
-		internal("buffer_to_16: unknown mem organization");
+			internal_error("buffer_to_16: unknown mem organization");
 #endif /* #ifdef DEBUG */
 
 	}
@@ -524,16 +487,16 @@ void buffer_to_bitmap_incremental(struct cached_image *cimg
 #ifdef DEBUG
 	if (cimg->state!=12&&cimg->state!=14){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in buffer_to_bitmap_incremental\n");
+		internal_error("Invalid state in buffer_to_bitmap_incremental\n");
 	}
 	if (height<1){
 		fprintf(stderr,"height=%d\n",height);
-		internal("Invalid height in buffer_to_bitmap_incremental\n");
+		internal_error("Invalid height in buffer_to_bitmap_incremental\n");
 	}
 	if (cimg->width<1||cimg->height<1){
 		fprintf(stderr,"cimg->width=%d, cimg->height=%d\n",cimg->width,
 				cimg->height);
-		internal("Invalid cimg->width x cimg->height in\
+		internal_error("Invalid cimg->width x cimg->height in\
 buffer_to_bitmap_incremental");
 	}
 #endif /* #ifdef DEBUG */
@@ -576,7 +539,6 @@ not_enough:
 	goto not_enough;
 end:
 	mem_free(tmp);
-	mem_freed_large(cimg->width*(height<max_height?height:max_height)*3*sizeof(*tmp));
 	if (!use_strip) drv->register_bitmap(&(cimg->bmp));
 }
 
@@ -596,13 +558,13 @@ static void buffer_to_bitmap(struct cached_image *cimg)
 #ifdef DEBUG
 	if(cimg->state!=12&&cimg->state!=14){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("buffer_to_bitmap called in invalid state");
+		internal_error("buffer_to_bitmap called in invalid state");
 	}
-	if (cimg->strip_optimized) internal("strip_optimized in buffer_to_bitmap");
+	if (cimg->strip_optimized) internal_error("strip_optimized in buffer_to_bitmap");
 	if (cimg->width<1||cimg->height<1){
 		fprintf(stderr,"cimg->width=%d, cimg->height=%d\n",cimg->width,
 				cimg->height);
-		internal("Invalid cimg->width x cimg->height in\
+		internal_error("Invalid cimg->width x cimg->height in\
 buffer_to_bitmap");
 	}
 #endif /* #ifdef DEBUG */
@@ -631,7 +593,7 @@ buffer_to_bitmap");
 		/* Scale the image to said size */
 #ifdef DEBUG
 		if (ox<=0||oy<=0){
-			internal("ox or oy <=0 before resampling image");
+			internal_error("ox or oy <=0 before resampling image");
 		}
 #endif /* #ifdef DEBUG */
 		if (tmp && (ix!=ox || iy!=oy)) {
@@ -647,7 +609,6 @@ buffer_to_bitmap");
 		if (!gonna_be_smart) {
 			if (tmp) {
 				mem_free(tmp);
-				mem_freed_large(ix*iy*3*sizeof(*tmp));
 			}
 		}
 		goto bitmap_failed;
@@ -665,11 +626,10 @@ buffer_to_bitmap");
 	}else{
 		if (tmp) {
 			if (dither_images)
-				dither(tmp,&(cimg->bmp));
+				dither(tmp, &cimg->bmp);
 			else
-				(*round_fn)(tmp,&(cimg->bmp));
+				(*round_fn)(tmp, &cimg->bmp);
 			mem_free(tmp);
-			mem_freed_large(ix*iy*3*sizeof(*tmp));
 		} else {
 			int i;
 			unsigned char *ptr = cimg->bmp.data;
@@ -719,7 +679,7 @@ void img_end(struct cached_image *cimg)
 #ifdef DEBUG
 		default:
 		fprintf(stderr,"state=%d\n",cimg->state);
-		internal("Invalid state encountered in end");
+		internal_error("Invalid state encountered in end");
 #endif /* #ifdef DEBUG */
 	}
 	cimg->state|=1;
@@ -776,7 +736,7 @@ static void r3l0ad(struct cached_image *cimg, struct g_object_image *goi)
 #ifdef DEBUG
 		default:
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in r3l0ad()");
+		internal_error("Invalid state in r3l0ad()");
 #endif /* #ifdef DEBUG */
 	}
 	cimg->state&=2;
@@ -796,7 +756,7 @@ static void type(struct cached_image *cimg, unsigned char *content_type, unsigne
 #ifdef DEBUG
 	if (cimg->state!=0&&cimg->state!=2){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state encountered in type()");
+		internal_error("Invalid state encountered in type()");
 	}
 #endif /* #ifdef DEBUG */
 #ifdef HAVE_JPEG
@@ -878,21 +838,22 @@ static void type(struct cached_image *cimg, unsigned char *content_type, unsigne
  */
 static int img_process_download(struct g_object_image *goi, struct f_data_c *fdatac)
 {
-	unsigned char *data, *dataend, *ctype;
-	int length;
+	unsigned char *data, *ctype;
+	size_t total_len;
 	struct cached_image *cimg = goi->cimg;
 	int chopped=0;
 
 #ifdef DEBUG
-	if (!goi->af) internal("NULL goi->af in process_download\n");
+	if (!goi->af) internal_error("NULL goi->af in process_download\n");
 	if (cimg->state>=16){ /* Negative don't occur because it's unsigned char */
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid cimg->state in img_process_download\n");
+		internal_error("Invalid cimg->state in img_process_download\n");
 	}
 #endif /* #ifdef DEBUG */
 	if (!goi->af->rq) return 0;
-	if (get_file(goi->af->rq, &data, &dataend)) goto end;
-	if (dataend - data < 4) goto end;
+	if (get_file(goi->af->rq, &data, &total_len)) goto end;
+	if (total_len < 4) goto end;
+	if (total_len > MAXINT) total_len = MAXINT;
 	/*fprintf(stderr, "processing: %s\n", goi->af->rq->ce->url);*/
 	if (goi->af->rq->ce->count2!=cimg->last_count2||
 		(goi->af->rq->ce->count!=cimg->last_count && cimg->eof_hit) ||
@@ -916,7 +877,7 @@ static int img_process_download(struct g_object_image *goi, struct f_data_c *fda
 	 */
 
 	if (!((cimg->state^8)&9)){
-		length = (int)(dataend - data);
+		int length = (int)total_len;
 		if (length<=cimg->last_length) goto end; /* No new data */
 
 		data+=cimg->last_length;
@@ -957,7 +918,7 @@ static int img_process_download(struct g_object_image *goi, struct f_data_c *fda
 #ifdef DEBUG
 		default:
 			fprintf(stderr,"cimg->image_type=%d\n",cimg->state);
-			internal("Invalid image_type encountered when processing data in\
+			internal_error("Invalid image_type encountered when processing data in\
 img_process_download.\n");
 #endif /* #ifdef DEBUG */
 		}
@@ -1016,7 +977,7 @@ int get_foreground(int rgb)
 static void draw_frame_mark(struct graphics_device *dev, int x, int y, int xw, int yw, long bg, long fg, int broken)
 {
 #ifdef DEBUG
-	if (xw<1||yw<1) internal("zero dimension in draw_frame_mark");
+	if (xw<1||yw<1) internal_error("zero dimension in draw_frame_mark");
 #endif /* #ifdef DEBUG */
 	if (broken == 1){
 		/* Draw between ( 0 and 1/4 ) and ( 3/4 and 1 ) of each
@@ -1074,39 +1035,54 @@ static void draw_frame_mark(struct graphics_device *dev, int x, int y, int xw, i
 	}
 }
 
+static long image_backgronud(struct cached_image *cimg)
+{
+	if (!cimg) return dip_get_color_sRGB(0x00c0c0c0);
+	return dip_get_color_sRGB(cimg->background_color);
+}
+
+static long image_foregronud(struct cached_image *cimg)
+{
+	if (!cimg) return dip_get_color_sRGB(0x00000000);
+	return dip_get_color_sRGB(get_foreground(cimg->background_color));
+}
+
 /* Entry is allowed only in states 12, 13, 14, 15
  * Draws the picture from bitmap.
  * Before doing so, ensures that bitmap is present and if not, converts it from
  * the buffer.
  */
-static void draw_picture(struct f_data_c *fdatac, struct g_object_image *goi,
-		int x, int y, long bg)
+static void draw_picture(struct f_data_c *fdatac, struct g_object_image *goi, int x, int y)
 {
-	struct graphics_device *dev=fdatac->ses->term->dev;
-	struct cached_image *cimg=goi->cimg;
+	struct graphics_device *dev = fdatac->ses->term->dev;
+	struct cached_image *cimg = goi->cimg;
 	struct rect saved;
 
 #ifdef DEBUG
-	if (goi->cimg->state<12||goi->cimg->state>=16){
-		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid cimg->state in draw_picture");
+	if (goi->cimg->state < 12 || goi->cimg->state >= 16) {
+		fprintf(stderr, "cimg->state=%d\n", cimg->state);
+		internal_error("Invalid cimg->state in draw_picture");
 	}
 #endif /* #ifdef DEBUG */
-	if (!(cimg->state&1)){
+	if (!(cimg->state & 1)) {
 		if (!cimg->bmp_used)
 			buffer_to_bitmap(cimg);
 	}
 #ifdef DEBUG
-	else if (!cimg->bmp_used){
-		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Nonexistent bitmap in said cimg->state in draw_picture");
+	else if (!cimg->bmp_used) {
+		fprintf(stderr, "cimg->state=%d\n", cimg->state);
+		internal_error("Nonexistent bitmap in said cimg->state in draw_picture");
 	}
 #endif /* #ifdef DEBUG */
-	restrict_clip_area(dev, &saved, x, y, x+goi->goti.go.xw, y+goi->goti.go.yw);
-	drv->draw_bitmap(dev, &cimg->bmp,x,y);
-	drv->fill_area(dev, x + cimg->bmp.x, y, x + goi->goti.go.xw, y + cimg->bmp.y, bg);
-	drv->fill_area(dev, x, y + cimg->bmp.y, x + goi->goti.go.xw, y + goi->goti.go.yw,bg);
-	drv->set_clip_area(dev, &saved);
+	restrict_clip_area(dev, &saved, x, y, x + goi->goti.go.xw, y + goi->goti.go.yw);
+	drv->draw_bitmap(dev, &cimg->bmp, x, y);
+	if (cimg->bmp.x < goi->goti.go.xw ||
+	    cimg->bmp.y < goi->goti.go.yw) {
+		long bg = image_backgronud(cimg);
+		drv->fill_area(dev, x + cimg->bmp.x, y, x + goi->goti.go.xw, y + cimg->bmp.y, bg);
+		drv->fill_area(dev, x, y + cimg->bmp.y, x + goi->goti.go.xw, y + goi->goti.go.yw, bg);
+	}
+	set_clip_area(dev, &saved);
 }
 
 /* Ensures in buffer there is not newer picture than in bitmap. Allowed to be
@@ -1115,21 +1091,19 @@ static void draw_picture(struct f_data_c *fdatac, struct g_object_image *goi,
 static void update_bitmap(struct cached_image *cimg)
 {
 #ifdef DEBUG
-	if (cimg->state<12||cimg->state>=16){
-		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in update_bitmap");
+	if (cimg->state < 12 || cimg->state >= 16) {
+		fprintf(stderr, "cimg->state=%d\n", cimg->state);
+		internal_error("Invalid state in update_bitmap");
 	}
 #endif /* #ifdef DEBUG */
-	if (!(cimg->state&1)&&
-		!cimg->strip_optimized
-		&&cimg->rows_added) buffer_to_bitmap(cimg);
+	if (!(cimg->state & 1) && !cimg->strip_optimized && cimg->rows_added)
+		buffer_to_bitmap(cimg);
 }
 
 /* Draws the image at x,y. Is called from other C sources. */
 static void img_draw_image(struct f_data_c *fdatac, struct g_object *goi_, int x, int y)
 {
 	struct g_object_image *goi = get_struct(goi_, struct g_object_image, goti.go);
-	long color_bg, color_fg;
 	struct cached_image *cimg = goi->cimg;
 	struct rect r;
 	/* refresh_image(fdatac, goi, 1000); To sem asi napsal mikulas jako
@@ -1137,21 +1111,14 @@ static void img_draw_image(struct f_data_c *fdatac, struct g_object *goi_, int x
 	 * usoudil, ze zadnejch 1000, ale 0.
 	 */
 
-	if (cimg) {
-		color_bg = dip_get_color_sRGB(cimg->background_color);
-		color_fg = dip_get_color_sRGB(get_foreground(cimg->background_color));
-	} else {
-		color_bg = dip_get_color_sRGB(0x00c0c0c0);
-		color_fg = dip_get_color_sRGB(0x00000000);
-	}
-
-	if (!(goi->goti.go.xw && goi->goti.go.yw)) return; /* At least one dimension is zero */
-
+	if (!(goi->goti.go.xw && goi->goti.go.yw))
+		return; /* At least one dimension is zero */
 
 	memcpy(&r, &fdatac->ses->term->dev->clip, sizeof(struct rect));
 	if (fdatac->vs->g_display_link && fdatac->active && fdatac->vs->current_link != -1 && fdatac->vs->current_link == goi->goti.link_num) {
-		draw_frame_mark(fdatac->ses->term->dev,x,y,goi->goti.go.xw,
-			goi->goti.go.yw,color_bg,color_fg,2);
+		long fg = image_foregronud(cimg);
+		draw_frame_mark(fdatac->ses->term->dev, x, y, goi->goti.go.xw,
+			goi->goti.go.yw, fg, fg, 2);
 		restrict_clip_area(fdatac->ses->term->dev, &r, x + 2, y + 2, x + goi->goti.go.xw - 2, y + goi->goti.go.yw - 2);
 	}
 
@@ -1160,26 +1127,21 @@ static void img_draw_image(struct f_data_c *fdatac, struct g_object *goi_, int x
 	if (img_process_download(goi, fdatac)) goto ret; /* Choked with data, will not
 							* draw. */
 	/* Now we will only draw... */
-	if (!cimg || cimg->state<12){
-		draw_frame_mark(fdatac->ses->term->dev,x,y,goi->goti.go.xw,
-			goi->goti.go.yw,color_bg,color_fg,!cimg || cimg->state&1);
-	}else
+	if (!cimg || cimg->state < 12) {
+		long fg = image_foregronud(cimg);
+		long bg = image_backgronud(cimg);
+		draw_frame_mark(fdatac->ses->term->dev, x, y, goi->goti.go.xw,
+			goi->goti.go.yw, bg, fg, !cimg || cimg->state & 1);
+	} else {
 #ifdef DEBUG
-	if (cimg->state<16){
-#else
-	{
-#endif /* #ifdef DEBUG */
+		if (cimg->state >= 16)
+			internal_error("Invalid state in img_draw_image: %d", cimg->state);
+#endif
 		update_bitmap(cimg);
-		draw_picture(fdatac,goi,x,y,color_bg);
+		draw_picture(fdatac, goi, x, y);
 	}
-#ifdef DEBUG
-	else{
-		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in img_draw_image");
-	}
-#endif /* #ifdef DEBUG */
-	ret:
-	drv->set_clip_area(fdatac->ses->term->dev, &r);
+ret:
+	set_clip_area(fdatac->ses->term->dev, &r);
 #ifdef LINKS_TESTMODE_IMAGE_AUTO_EXIT
 	if (cimg->state & 1)
 		terminate_loop = 1;
@@ -1207,7 +1169,7 @@ static void find_or_make_cached_image(struct g_object_image *image, unsigned cha
 		cimg->background_color = image->background;
 #ifdef DEBUG
 		if (!url)
-			internal ("NULL url as argument of\
+			internal_error("NULL url as argument of\
 find_or_make_cached_image");
 #endif /* #ifdef DEBUG */
 		cimg->scale = scale;
@@ -1302,7 +1264,7 @@ struct g_object_image *insert_image(struct g_part *p, struct image_description *
 	*/
 	image->af = request_additional_file(current_f_data, im->url);
 	if (image->goti.go.xw < 0 || image->goti.go.yw < 0) image->af->unknown_image_size = 1;
-	image->background = hack_rgb(p->root->bg->u.sRGB);
+	image->background = hack_rgb(p->root->bg->sRGB);
 
 	/* This supplies the result into image->cimg and global_cimg */
 	find_or_make_cached_image(image, im->url, d_opt->image_scale);
@@ -1332,7 +1294,7 @@ void change_image (struct g_object_image *goi, unsigned char *url, unsigned char
 	goi->orig_src = stracpy(src);
 	if (!(goi->goti.go.xw && goi->goti.go.yw)) return;
 	goi->cimg->refcount--;
-	goi->af = request_additional_file(fdata,url);
+	goi->af = request_additional_file(fdata, url);
 	goi->af->need_reparse = -1;
 
 	find_or_make_cached_image(goi, url, fdata->opt.image_scale);
